#!/usr/bin/perl 
##########################################################################
#                                                                        #
# Perl Script for check mysql replication                                #
# Switzernet(c)2011                                                      #
#                                                                        #
##########################################################################
# my modules
use strict;
use warnings;
#use diagnostics;
use POSIX;
use Config::IniFiles;
use Switch;
use DBI;
use Date::Manip;
use MIME::Lite;
##########################################################################
# Setting
my $config_file = '/root/folders/110726-check-replication/check-rep.conf';
##########################################################################
# Globals
my ($DB_LOC_NAME,$DB_LOC_USER,$DB_LOC_PASS,$DB_LOC_TAB);
my %LEG;
my @Mailto;
##########################################################################
#                             Initialization                             #
##########################################################################
# Remove leading and trailing white space from a string
sub chomp_plus {
my $string=shift;
$string =~ s/^\s+//;$string =~ s/\s+$//;
chomp $string;
return $string;
}
##########################################################################
# Load config from mysql.conf
sub load_config {
my $conf=Config::IniFiles->new(-file => $config_file);
return 0 if !defined $conf ;
@Mailto = split(',', $conf->val('GLOBAL','Mailto'));
$DB_LOC_NAME = $conf->val('GLOBAL','DBname');
$DB_LOC_USER = $conf->val('GLOBAL','Luser');
$DB_LOC_PASS = $conf->val('GLOBAL','Lpass');
$DB_LOC_TAB  = $conf->val('GLOBAL','Table');
my @LEGS = split(',', $conf->val('GLOBAL','Stars'));
foreach my $i (@LEGS) {
   my $j=chomp_plus($i);
   $LEG{$j}{'Master'}=$conf->val($j,'Master');
   $LEG{$j}{'Muser'}=$conf->val($j,'Muser');
   $LEG{$j}{'Mpass'}=$conf->val($j,'Mpass');
   my @tmp1 = split(',', $conf->val($j,'Slaves'));
   $LEG{$j}{'Slaves'}=\@tmp1;
   $LEG{$j}{'Suser'}=$conf->val($j,'Suser');
   $LEG{$j}{'Spass'}=$conf->val($j,'Spass');
   $LEG{$j}{'DBname'}=$conf->val($j,'DBname');
   my @tmp2 = split(',', $conf->val($j,'Tables'));
   $LEG{$j}{'Tables'}=\@tmp2;
   $LEG{$j}{'Rep_status'}=$conf->val($j,'Rep_status');
   $LEG{$j}{'Count_table'}=$conf->val($j,'Count_table');
   $LEG{$j}{'Sum_table'}=$conf->val($j,'Sum_table');
   }
return 1;
}
##########################################################################
# DB connect 
sub DB_connect {
my($DB_NAME,$DB_HOST,$DB_USER,$DB_PASS) = @_;
my $cop=0;
my $tdbh;
while( 1 ) {
        $cop++;
        $tdbh = DBI->connect_cached("DBI:mysql:$DB_NAME;host=$DB_HOST", $DB_USER, $DB_PASS);
        if( $tdbh ) {
	   return $tdbh;
           } 
        else {
           return 0 if ($cop>=3);
	   }
        sleep $cop;
	}
}
##########################################################################
# checksum table
sub get_status_master {
my $dbh=shift;
my $query = "show master status";
my $sqlQuery  = $dbh->prepare($query) or return ;
$sqlQuery->execute or return;
my @row= $sqlQuery->fetchrow_array();
$sqlQuery->finish;
return ($row[0],$row[1])
}
#########################################################################
# checksum table
sub get_status_slave  {
my $dbh=shift;
my $query = "show slave status";
my $sqlQuery  = $dbh->prepare($query) or return;
$sqlQuery->execute or return;
my @row= $sqlQuery->fetchrow_array();
$sqlQuery->finish;
return ($row[5],$row[6],$row[0],$row[18],$row[19])
}
#########################################################################
# checksum table
sub get_count {
my ($dbh,$tab)=@_;
my %vr;
foreach my $t (@$tab) {
   my $query = "SELECT COUNT(*) FROM $t";
   my $sqlQuery  = $dbh->prepare($query) or return;
   $sqlQuery->execute or return;
   my @row= $sqlQuery->fetchrow_array();
   $sqlQuery->finish;
   $vr{$t}=$row[0];
   }
return %vr;
}
#########################################################################
# checksum table
sub get_sum {
my ($dbh,$tab)=@_;
my %vr;
foreach my $t (@$tab) {
   my $query = "CHECKSUM TABLE $t";
   my $sqlQuery  = $dbh->prepare($query) or return;
   $sqlQuery->execute or return;
   my @row= $sqlQuery->fetchrow_array();
   $sqlQuery->finish;
   $vr{$t}=$row[1];
   }
return %vr;
}
##########################################################################
# sample '2004-12-11 09:16:05'
sub format_date_sql {
    my ($date) = @_;
    if ($date eq 0) {
        return "NULL";
    } else {
        return strftime("%Y-%m-%d %H:%M:%S", localtime($date));
    }
}
##########################################################################
# Send mail
sub Send_mail {
#print $Mailto;
my $body=shift;
my $date=format_date_sql(time);
foreach my $to (@Mailto) {
my $Message = new MIME::Lite
   From =>'mysql-replicatiom-monitor <noreply@switzernet.com>',
   To =>$to,
   Subject =>'Error check-replication: '.$date,
   Type =>'TEXT',
   Data =>"Date: '$date' \n$body \n\n\t\t\t\t~ Switzernet(c)2011 ~\n";
$Message -> send;
}
}
##########################################################################
#                                 main()                                 #
##########################################################################
load_config();
#$DBH_LOC=DB_connect($DB_NAME_LOC,'localhost',$DB_USER_LOC);
foreach my $j (keys %LEG) {
   my $key = "";
   my %req;
   my %dbh;
   my %body;
   my %status;
   my $ref=$LEG{$j}{'Slaves'};
   $dbh{'Master'}=DB_connect($LEG{$j}{'DBname'},$LEG{$j}{'Master'},$LEG{$j}{'Muser'},$LEG{$j}{'Mpass'});
   foreach my $i (@$ref) {
      $dbh{$i}=DB_connect($LEG{$j}{'DBname'},$i,$LEG{$j}{'Suser'},$LEG{$j}{'Spass'});
      $req{$i}="INSERT INTO $DB_LOC_TAB SET date=NOW(), master='".$LEG{$j}{'Master'}."', slave='$i', ";
      $body{$i}="Master = '".$LEG{$j}{'Master'}."'\nSlave  = '$i'\n";
      $status{$i}=1;
      }
   if ($dbh{'Master'}){
      if ($LEG{$j}{'Rep_status'} eq 'Y') {
         my ($M_Log_File,$M_Log_Pos)=get_status_master($dbh{'Master'});
         foreach my $i (@$ref) {
            if ($dbh{$i}) {
               my ($S_Log_File,$S_Log_Pos,$Slave_IO_State,$Last_Errno,$Last_Error)=get_status_slave($dbh{$i});
               $body{$i}.="Master_Log_File = '$M_Log_File'\n";
               $body{$i}.="Master_Log_Pos = '$M_Log_Pos'\n";
               $body{$i}.="Slave_Log_File = '$S_Log_File'\n";
               $body{$i}.="Slave_Log_Pos = '$S_Log_Pos'\n";
               $body{$i}.="Slave_IO_State = '$Slave_IO_State'\n";
               $body{$i}.="Slave_Last_Errno = '$Last_Errno'\n";
               $body{$i}.="Slave_Last_Error = '$Last_Error'\n";
               $req{$i}.="Master_Log_File='$M_Log_File', ";
               $req{$i}.="Master_Log_Pos='$M_Log_Pos', ";
               $req{$i}.="Slave_Log_File='$S_Log_File', ";
               $req{$i}.="Slave_Log_Pos='$S_Log_Pos', ";
               $req{$i}.="Slave_IO_State='$Slave_IO_State', ";
               $req{$i}.="Slave_Last_Errno='$Last_Errno', ";
               $req{$i}.="Slave_Last_Error='$Last_Error', ";
               $status{$i}=2 if ($M_Log_Pos != $S_Log_Pos || $Last_Error );
               $status{$i}=3 if ($M_Log_File ne $S_Log_File || $Last_Errno || !$Slave_IO_State);
               }
            }
         }

      if ($LEG{$j}{'Count_table'} eq 'Y') {
         my %M_count=get_count($dbh{'Master'},$LEG{$j}{'Tables'});
         foreach my $i (@$ref) {
            if ($dbh{$i}) {
               my %S_count=get_count($dbh{$i},$LEG{$j}{'Tables'});
               $req{$i}.= "count_tab='";
               $body{$i}.= "count_tab='";
               foreach my $k (keys %S_count) {
                  $req{$i}.= "Master_$k=$M_count{$k};Slave_$k=$S_count{$k};";
                  $body{$i}.= "Master_$k=$M_count{$k};Slave_$k=$S_count{$k};";
                  $status{$i}=3 if ($M_count{$k} != $S_count{$k});
                  }
               $req{$i} = substr($req{$i},0,length($req{$i})-1)."', ";
               $body{$i} = substr($body{$i},0,length($body{$i})-1)."'\n";
               }
            }
         }

      if ($LEG{$j}{'Sum_table'} eq 'Y') {
         my %M_sum=get_sum($dbh{'Master'},$LEG{$j}{'Tables'});
         foreach my $i (@$ref) {
            if ($dbh{$i}) {
               my %S_sum=get_sum($dbh{$i},$LEG{$j}{'Tables'});
               $req{$i}.= "sum_tab='";
               $body{$i} .= "sum_tab='";
               foreach my $k (keys %S_sum) {
                  $req{$i}.= "Master_$k=$M_sum{$k};Slave_$k=$S_sum{$k};";
                  $body{$i}.= "Master_$k=$M_sum{$k};Slave_$k=$S_sum{$k};"; 
                  $status{$i}=3 if ($M_sum{$k} != $S_sum{$k});
                  }
               $req{$i} = substr($req{$i},0,length($req{$i})-1)."', ";
               $body{$i} = substr($body{$i},0,length($body{$i})-1)."'\n ";
               }
            }
         }
      $dbh{'Master'}->disconnect ;
   }
   my $ins=DB_connect($DB_LOC_NAME,'localhost',$DB_LOC_USER,$DB_LOC_PASS);
   foreach my $i (@$ref) {
      if ($dbh{$i}) {
         $dbh{$i}->disconnect;
         $req{$i}.= "status=$status{$i}";
         Send_mail($body{$i}) if ($status{$i}!=1);
         $ins->do($req{$i});
         }
      }
   $ins->disconnect ;
   }
##########################################################################
#                                  END                                   #
##########################################################################
